package pathfinder

import (
	"fmt"
	"reflect"
	"strings"
)

func FindPaths(v reflect.Value, targetType reflect.Type, path []string, paths *[]string) {

	if v.Kind() == reflect.Invalid {
		return
	}

	vType := v.Type()

	switch v.Kind() {
	case reflect.Ptr:
		FindPaths(v.Elem(), targetType, path, paths)
	case reflect.Struct:
		for i := 0; i < v.NumField(); i++ {
			newPath := append(path, vType.Field(i).Name)
			FindPaths(v.Field(i), targetType, newPath, paths)
		}
	case reflect.Map:
		for _, key := range v.MapKeys() {
			newPath := append(path, fmt.Sprint(key))
			FindPaths(v.MapIndex(key), targetType, newPath, paths)
		}
	case reflect.Slice, reflect.Array:
		for i := 0; i < v.Len(); i++ {
			newPath := append(path, fmt.Sprint(i))
			FindPaths(v.Index(i), targetType, newPath, paths)
		}

	case reflect.String, reflect.Bool, reflect.Int64, reflect.Int32, reflect.Int16, reflect.Int8, reflect.Int, reflect.Uint64, reflect.Uint32, reflect.Uint16, reflect.Uint8, reflect.Uint:
		if vType != targetType {
			return
		}

	default:
		return
	}

	if vType == targetType {
		*paths = append(*paths, strings.Join(path, "."))
	}
}
